import java.util.Map;
import java.util.HashMap;
import java.util.PriorityQueue;
import java.util.Comparator;

import bridges.base.Element;
import bridges.base.SLelement;
import bridges.base.GraphAdjList;
import bridges.base.Edge;
import bridges.base.Color;
import bridges.base.ElementVisualizer;
import bridges.base.LinkVisualizer;
import bridges.connect.Bridges;
import bridges.data_src_dependent.OsmData;
import bridges.data_src_dependent.OsmVertex;
import bridges.data_src_dependent.OsmEdge;

//class useful as what get pushed in Dijkstra's priority queue
class Pair<Vtx, Dst> {
    Vtx v;
    Dst d;

    Pair(Vtx ve, Dst di) {
	v=ve;
	d=di;
    }

    Dst getValue() {
	return d;
    }

    Vtx getKey() {
	return v;
    }
}

//Comparator use in Dijkstra priority queue
class PQ_Comparator implements Comparator<Pair<Integer, Double>>{

    // Overriding compare()method of Comparator for descending order
    public int compare(Pair<Integer,Double> p1, Pair<Integer,Double> p2) {
	double d1 = p1.getValue().doubleValue();
	if (p1.getValue().doubleValue() < p2.getValue().doubleValue())
	    return 1;
	else if (p1.getValue().doubleValue() > p2.getValue().doubleValue())
	    return -1;
	return 0;
    }
}




public class osm {

    //returns the relative difference between two double. Useful to
    //compare equality between doubles.
    public static double RelDif(double a, double b)
    {
	double c = Math.abs(a);
	double d = Math.abs(b);

	d = Math.max(c, d);

	return d == 0.0 ? 0.0 : Math.abs(a - b) / d;
    }


    //dijsktra's shortest path algorithm 
    public static void dijkstra (GraphAdjList<Integer, OsmVertex, Double> gr,
			  int source,
			  HashMap<Integer, Double> distance) {

	for (Map.Entry<Integer, Element<OsmVertex> > pai : gr.getVertices().entrySet())
	    distance.put (pai.getKey(), Double.MAX_VALUE);

	distance.put(source, 0.);

	//using a pair<vertex, distance> in the queue rather than just vertex
	//becasue I am not sure how the heap is going to like the vertices changing silently distance.
	//should really use a fibonacci heap.
  
	PriorityQueue<Pair<Integer, Double>> pq = new PriorityQueue<Pair<Integer, Double>> (100, new PQ_Comparator());
  
	pq.add(new Pair<Integer, Double>(source, distance.get(source)));

	//as long as the queue is not empty
	while (!pq.isEmpty()) {
	    int from = pq.peek().getKey();
	    pq.remove();

	    double d_from = distance.get(from);

	    //iterate over neighbors
	    SLelement<Edge<Integer, Double>> list = gr.getAdjacencyList(from);
	    while (list != null) {
		int  to = list.getValue().getVertex();
		Double ddd = list.getValue().getEdgeData();
		double edgelength = ddd;
      
		double pathlength = d_from + edgelength;
		if (pathlength < distance.get(to)) { //new shortest path found
		    distance.put(to, pathlength);
		    pq.add(new Pair<Integer, Double>(to, pathlength));
		}
      
		list = list.getNext();
	    }
	}  
    }


    // return the vertex from the graph that is the closest to (lat,lon)
    public static int getClosestVertex(GraphAdjList<Integer, OsmVertex, Double> graph, double lat, double lon) {
	//TODO
	return -1;
    }

    //this function is used to style vertices based on their distance to the source
    public static void styleDistance(GraphAdjList<Integer, OsmVertex, Double> graph,
				HashMap<Integer, Double> distance) {
	//TODO

	//Find max distance. (Beware of unreachable vertices with a distance of Double.MAX_VALUE)

	//color vertices based on distances

	//optional: style edges if they are shortest path edges. (Beware of back edges)
	
    }

    //style the graph by greying out all vertices and edges except
    //those on the SP between the source and destinatin. (No need to
    //pass the source since all parent chase go there.)
    public static void styleParent(GraphAdjList<Integer, OsmVertex, Double> graph,
			    HashMap<Integer, Double> distance,
			    HashMap<Integer, Integer> parent,
			    int dest
			    ) {
	//TODO

	//set all edges to transparent
 
	//set all vertices to transparent

	//for each edge on the SP from source to dest
	////Style vertex on the path

	////style edge on the path (beware of potential back edges)
    }

    //style the root of the shortest path
    public static void styleRoot(GraphAdjList<Integer, OsmVertex, Double> graph,
			  int root) {
	//TODO
    }

    public static void main(String[] args) throws Exception {
  
	//create the Bridges object, set credentials 
	Bridges bridges = new Bridges(Integer.parseInt(args[0]), args[1], args[2]);
	bridges.setTitle("Graph : OpenStreet Map Example");
	bridges.setServer("clone");
  
	//Getting Data
	int closest;
	double latc, lonc;
	int dest;

	//TODO: Get data!

	////TODO: Uncomment for part 2
	// double[] latr = new double[2];
	// double[] lonr = new double[2];
	// osm_data.getLatLongRange(latr, lonr);
	////Getting source vertex (Using center of the map)
	// closest = getClosestVertex(graph,
	// 			   latr[0]+(latr[1]-latr[0])/2.,
	// 			   lonr[0]+(lonr[1]-lonr[0])/2.);
	
	// dest = getClosestVertex(graph,
	// 			latr[0]+(latr[1]-latr[0])/4.,
	// 			lonr[0]+(lonr[1]-lonr[0])/4.);
	// styleRoot(graph, closest);
	// bridges.setDataStructure(graph);
	// bridges.visualize();

	//TODO: Uncomment for part 3
	////Running shortest path
	//HashMap<Integer, Double> distance= new HashMap<Integer, Double>();
	////HashMap<Integer, Integer> parent = new HashMap<Integer, Integer>();
	//dijkstra(graph, closest, distance);
	////dijkstra(graph, closest, distance, parent);

	////Styling
	// styleDistance(graph, distance);
	// bridges.visualize();

	//TODO: Uncomment for part 4
	// styleParent(graph, distance, parent, dest);
	// bridges.visualize();
    }

}

